(**************************************************************************)
(* Copyright (c) 2010, Romain BARDOU                                      *)
(* All rights reserved.                                                   *)
(*                                                                        *)
(* Redistribution and  use in  source and binary  forms, with  or without *)
(* modification, are permitted provided that the following conditions are *)
(* met:                                                                   *)
(*                                                                        *)
(* * Redistributions  of  source code  must  retain  the above  copyright *)
(*   notice, this list of conditions and the following disclaimer.        *)
(* * Redistributions in  binary form  must reproduce the  above copyright *)
(*   notice, this list of conditions  and the following disclaimer in the *)
(*   documentation and/or other materials provided with the distribution. *)
(* * Neither the  name of Capucine nor  the names of its contributors may *)
(*   be used  to endorse or  promote products derived  from this software *)
(*   without specific prior written permission.                           *)
(*                                                                        *)
(* THIS SOFTWARE  IS PROVIDED BY  THE COPYRIGHT HOLDERS  AND CONTRIBUTORS *)
(* "AS  IS" AND  ANY EXPRESS  OR IMPLIED  WARRANTIES, INCLUDING,  BUT NOT *)
(* LIMITED TO, THE IMPLIED  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR *)
(* A PARTICULAR PURPOSE  ARE DISCLAIMED. IN NO EVENT  SHALL THE COPYRIGHT *)
(* OWNER OR CONTRIBUTORS BE  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, *)
(* SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES (INCLUDING,  BUT  NOT *)
(* LIMITED TO, PROCUREMENT OF SUBSTITUTE  GOODS OR SERVICES; LOSS OF USE, *)
(* DATA, OR PROFITS; OR BUSINESS  INTERRUPTION) HOWEVER CAUSED AND ON ANY *)
(* THEORY OF  LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY,  OR TORT *)
(* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING  IN ANY WAY OUT OF THE USE *)
(* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   *)
(**************************************************************************)

type 'a t = (Ident.t * 'a) list

let empty = []

let add env x v = (x, v) :: env

let remove env x = try List.remove_assoc x env with Not_found -> env

let replace env x v =
  add (remove env x) x v

let find env x = List.assoc x env

let mem env x = List.mem_assoc x env

let lookup env s =
  fst (List.find (fun (x, _) -> Ident.name x = s) env)

let mem_string env s =
  List.exists (fun (x, _) -> Ident.name x = s) env

let lookup_and_find env s =
  List.find (fun (x, _) -> Ident.name x = s) env

let add_list env l =
  List.fold_left (fun env (a, b) -> add env a b) env l

let rec such_as env f =
  match env with
    | [] ->
        raise Not_found
    | (id, x) :: rem ->
        if f x then
          id
        else
          such_as rem f

let filter env f =
  List.map fst (List.filter (fun (_, x) -> f x) env)

open Format

let pp f fmt env =
  match env with
    | [] -> fprintf fmt "[]"
    | (x, y) :: r ->
        fprintf fmt "[@[<hov 2>%a: %a" Ident.upp x f y;
        List.iter (fun (x, y) -> fprintf fmt ",@ %a: %a" Ident.upp x f y) r;
        fprintf fmt "@]]"
